# Definition of our song framework.
# Should run before the"init" block where we use the framework to define the songs.
init -1 python:
    
    TM_song_list = []
    
    # A dictionary of True/False variables, holding unlocked status of each song.
    # (Note: put this in an init block, not an init -1 block)
    if persistent.bgm_unlocked is None:
        persistent.bgm_unlocked = {}
    
    # Put current_bgm in store, so that the current_bgm is part of Ren'Py's
    # game context.  Thus, it interacts well with rollback and saving/loading of games.
    # HOWEVER, it won't interact very well with the title screen, so be careful
    # with your title screen / main menu code.
    # (Same behavior is observed if you don't put "store."  But better to be explicit, I guess?)
    #
    # Note: if you put this as a class variable, TMsong.current_bgm, it'll interact
    # well with the title screen.  However, it won't interact well with rollback; in
    # particular, when we switch from playing one song to another song.  Rolling back
    # to when the old song was playing, and then advancing again to when the new song
    # should play, will not make the new song re-play.  We need to make sure current_bgm
    # is in the current game's "context" so it plays nice with rollback.
    store.current_bgm = None
    
    class TMsong:
        
        # mainloop: filename of the loop section of the song. *Required*
        # intro: filename of the intro section of the song.  *Optional*
        # title: a title for the song (a string). *Required for music room to work*
        # description: a description of the song (a string).  *Best to have this for music room*
        
        def __init__(self, mainloop, intro=None, title=None, author=None, description=None, loops=True, musicroom=True):
            
            # Nothing special, just processing arguments
            if not isinstance(mainloop, basestring):
                raise Exception("The mainloop parameter, %s, must be a filename string." % mainloop)
            if (intro is not None) and (not isinstance(intro, basestring)):
                raise Exception("The intro parameter, %s, must be a filename string." % intro)
            if (title is not None) and (not isinstance(title, basestring)):
                raise Exception("The title parameter, %s, must be a string." % title)
            if (author is not None) and (not isinstance(author, basestring)):
                raise Exception("The title parameter, %s, must be a string." % author)
            if (description is not None) and (not isinstance(description, basestring)):
                raise Exception("The description parameter, %s, must be a string." % description)
            if (not isinstance(loops, bool)):
                raise Exception("The loops parameter, %s, must be True or False." % loops)
            if (not isinstance(musicroom, bool)):
                raise Exception("The musicroom parameter, %s, must be True or False." % musicroom)
            self.mainloop = mainloop
            self.intro = intro
            self.title = title
            self.author = author
            self.description = description
            self.loops = loops
            self.musicroom = musicroom
            
            if self.musicroom:
            
                # Add the song to the song list, so the music room can
                # retrieve it
                TM_song_list.append(self)
                
                # If this is a newly added song, initialize unlocked status to False
                if not persistent.bgm_unlocked.has_key(self.title):
                    persistent.bgm_unlocked[self.title] = False
            
            
        # The staticmethod decorator allows us to call the method by the
        # class name, instead of with an instance of the class.
        # This also means the method doesn't have a "self" parameter.
        
        @staticmethod
        def isPlaying(title):
            
            # Ren'Py's "stop music" doesn't reset current_bgm,
            # so we need to check whether any music is playing by using
            # renpy.music.get_playing()
            
            return ( store.current_bgm == title and renpy.music.get_playing() )
            
        def isUnlocked(self):
            
            return persistent.bgm_unlocked[self.title]
            
        def play(self, fadeout=0.5, fadein=0.0):
            
            # If this song's already playing, do nothing.
            if TMsong.isPlaying(self.title):
                return
            
            # Otherwise, start playing the song
            if self.intro is None:
                renpy.music.play(self.mainloop, fadeout=fadeout, fadein=fadein, loop=self.loops)  # This has to be play, not queue, so we can replace any currently-playing music.
            else:
                renpy.music.play(self.intro, fadeout=fadeout, fadein=fadein)
                renpy.music.queue(self.mainloop, loop=self.loops)
                
            store.current_bgm = self.title
            
            if self.musicroom:
                # Unlock the song if it's not already unlocked
                if persistent.bgm_unlocked[self.title] == False:
                    persistent.bgm_unlocked[self.title] = True
            
        @staticmethod
        def stop(fadeout=0.0):
            
            renpy.music.stop(fadeout=fadeout)
            store.current_bgm = None
                
            # You can still use Ren'Py's "stop music" statement in the story script,
            # because even if current_bgm is not reset, the isPlaying() method
            # keeps the play() behavior correct by checking whether anything is
            # playing.
            # This stop() method is mainly for the music room, where the stop() method
            # helps to make timely updates to the "selected" state of a button.
            
            # Note that whichever way you stop the music, you'll run into odd behavior
            # if you switch to yet another song while the fadeout from a previous song switch
            # is going on.  The odd behavior is that one song will just continually fade in
            # and out as you're continually and quickly switching songs.
                
                
               
## ------------------------ ##
## --- Song definitions --- ##
## ------------------------ ##
  
init python:
        
        # NOTE: Song titles (the "title" field) must be unique.
        
        
        # The main menu theme received the "combine fadeout with first part of loop"
        # treatment, but we still have one .ogg file for it instead of an intro file
        # + loop file.  This is due to complications with playing the main menu
        # theme in particular.
        # Fortunately, having combine fade + one file does not have a strong side
        # effect on the start of the song in the main menu theme's case.
        
        #Chapter 1
        
        bgm_title = TMsong("music/oc-tm-title.ogg",
                               title="Title",
                               author="OverCoat",
                               musicroom=True,
                               description="It's the title jingle!")
        bgm_overture = TMsong("music/oc-tm-overture.ogg",
                                title="Overture",
                               author="OverCoat",
                               description="I really like Yamame, and I think her mech is cool. Too bad it breaks so fast. Since she's the villain in the fight, I decided to use \"The Sealed Away Youkai ~ Lost Place\" as a base to start the song. I also wanted it to be loud and chaotic, since it's the start of the story, so I made this.")
        bgm_lolz = TMsong("music/oc-tm-lolz-softer.ogg",
                               title="Lolz",
                               author="OverCoat",
                               description="You'll know Okuu is talking and being her birdbrained self when this song plays. This was the first song I wrote for the game, trying to capture the feeling of Utsuho and her less-than-serious interactions with her friend Rin and others. I had the hardest time writing this and so it ended up going through several revisions before landing at this final version, trying not to annoy the reader too much.")
        bgm_business = TMsong("music/oc-tm-business.mp3",
                               title="Business",
                               author="OverCoat",
                               description="A more normal conversation song. It's quite short, but that's okay, there is very little time for talking and lots of time for lasers!")
        bgm_probing = TMsong("music/oc-tm-probing.mp3",
                               title="Probing",
                               author="OverCoat",
                               description="This is the song you hear when the heroes of our story capture some of the bad guys. I am assuming Yamame was too full-bodied to escape her cockpit. Poor Yamame! You will hear this theme again in later chapters, in a more fleshed out and evil tone.")
        bgm_parsia = TMsong("music/oc-tm-parsia.mp3",
                               title="Parsia",
                               author="OverCoat",
                               description="So for this theme, I tried to convey Parsee's emotions. It's dark, and distorted, and a little bit sad. This was one of the first songs I completed, and one of the songs I think turned out the best!")
        bgm_hellfire_surprise = TMsong("music/oc-tm-hellfiresurprise.ogg",
                               title="Hellfire Surprise",
                               author="OverCoat",
                               description="This was one of the first songs I wanted to write - a battle theme with a sense of urgency. For this song I felt I'd draw from Hellfire Mantle, it already had the Subterranean Animism connection, and the proper mood and tempo for an event such as a battle.")
        bgm_vision = TMsong("music/oc-tm-vision.mp3",
                               title="Vision",
                               author="OverCoat",
                               description="I went fully ambient electronic on this one, to stress how alien and intangible the sun god feels to the rest of the characters in the story. The Yatagarasu is a god without physical form, at least until somebody ate him, so I created something without percussion. Rather, at this point, the percussion is controlled by the reader, at their own pace.")
        bgm_altered_beast = TMsong("music/oc-tm-alteredbeast.mp3",
                               title="Altered Beast",
                               author="OverCoat",
                               description="The \"Powerup theme\" for Utsuho, the main character of this story! It ended up being a mash of different themes from Subterranean Animism, with a little bit of an original take on them so they melded better.")
        bgm_helios_1 = TMsong("music/Helios Sample 1.MP3",
                               title="Helios 1",
                               author="Flamezero16",
                               description="No comment")
        bgm_helios_2 = TMsong("music/Helios Sample 2.MP3",
                               title="Helios 2",
                               author="Flamezero16",
                               description="No comment")
        bgm_preamble = TMsong("music/oc-tm-preamble.mp3",
                               title="Preamble",
                               author="OverCoat",
                               description="I had a lot of fun writing this one. I melded together several different worlds, which goes along with the theme of this game, both stylistically and canonically. I started off with the typical eastern Touhou sound, moving to orchestral, and then going full synth later on. I even threw in a Terminator reference. This is the song that sets up the rest of the story, so I wanted to make a beautiful song with flowing string pads and also a sense of foreboding.")
        bgm_library = TMsong("music/oc-tm-library.mp3",
                               title="Library",
                               author="OverCoat",
                               musicroom=True,
                               description="This song encompasses all the data available in the game, so I had to make it lengthy since readers should read the whole database! I wanted to cover a wide range of emotions but instead I ended up making a medley of sorts with discernable sections. What starts as a serious delving into the lore becomes an emotional journey into the characters and setting. I have no idea why but I was inspired by Command & Conquer on this song. Haha!")
        
        #Chapter 2
                               
        bgm_heavy_duty = TMsong("music/oc-tm-heavyduty.mp3",
                               title="Heavy Duty",
                               author="OverCoat",
                               musicroom=True,
                               description="This song is a short intro for Yuugi's cool mech. It looks kind of like an Atlas, right? It's huge, slow, and badass, so I tried to convey that for her introduction to the battlefield.")
        
                               

# Step 3. Create the music room screen.
screen music_room:

    tag menu

    on "hide" action Hide("music_room_info")
    on "replaced" action Hide("music_room_info")    # Prevent info text from staying onscreen when going back to main menu.
    
    imagemap:
        alpha False
        
        ground MusicRoomImages.ground
        hover MusicRoomImages.hover
        
        hotspot (67, 522, 127, 40):
            action Return()
    
    text "Click on a song title in the left box to":
        style "footnote_text"
        pos(358, 512)
    text "play the song.":
        style "footnote_text"
        pos(358, 532)
            
            
            
    # Main window
    
    $ BUTTON_SIZE = (332, 36)
    
    window:
        area (15, 100, 415, 372)
        xpadding 25 ypadding 0
        background None
        
        viewport id "music_room_viewport":
            yadjustment initialize_adjustment(
                "music_room_viewport",
                persistent_save=True,
            )

            mousewheel True
                
            vbox:
                style_group "song_list"
                for i in range(len(TM_song_list)):
                    $ song = TM_song_list[i]
                    $ title = song.title

                    if config.developer or song.isUnlocked():
                        textbutton title clicked PlayOrStopSong(song):
                            hovered Show("music_room_info", author_name=song.author, song_title=song.title, song_description=song.description)
                            minimum BUTTON_SIZE
                            maximum BUTTON_SIZE
                            
    # Main window's scrollbar
    vbox:
        xpos 27 ypos 100

        # Amount scrolled when the up or down button is clicked.  Make it equal to 3 lines.
        $ scroll_amount = 78

        style_group "music_room_scrollbar"

        # TODO: perhaps make hovered versions of the up/down buttons.
        # TODO: make holding down the mouse on the up/down buttons scroll repeatedly.
        button action ScrollUp("music_room_viewport", scroll_amount):
            style "music_room_scrollbar_arrow_up"

        vbar value YScrollValue("music_room_viewport")

        button action ScrollDown("music_room_viewport", scroll_amount):
            style "music_room_scrollbar_arrow_down"
                            

    
screen music_room_info:
    
    window background None:
        area (450, 159, 500, 42)
        xpadding 0 ypadding 0
        
        text author_name:
            style "description_text"
    
    window background None:
        area (450, 257, 500, 270)
        xpadding 0 ypadding 0
        
        text song_description:
            style "description_text"
            
            
    
init python:
    
    # These names don't conflict with anything in Ren'Py, for now (as of 6.12.0e)
    
    
    class StopMusic:
        
        def __call__(self):
            TMsong.stop(fadeout=1.0)
    
    class PlayOrStopSong(Action):
        
        def __init__(self, song):
            self.song = song
            
        def __call__(self):
            # if current song then stop, if not current song then play
            if TMsong.isPlaying(self.song.title):
                TMsong.stop(fadeout=1.0)
            else:
                self.song.play()
            renpy.restart_interaction()
            
        # The get_selected function, along with the fact that this class
        # inherits from the class Action, lets us style a music room button
        # differently when that button's song is playing.
        
        def get_selected(self):
            return TMsong.isPlaying(self.song.title)
        
            
    
    class MusicRoomImages(object):
        """
        Filenames of images used in the chapter select screens.
        """
        prefix = "gui_images/music_room/"
        
        ground = prefix + "music_room_ground.png"
        hover = prefix + "music_room_button_hovered.png"
        music_room_songtitle = prefix + "music_room_songtitle.png"
    
    # Stylings for the Music Room.
    style.song_list_vbox = Style(style.default, properties=dict(
    ))
    style.song_list_button = Style(style.default, properties=dict(
        idle_background = None,
        hover_background = None,
        selected_idle_background = MusicRoomImages.music_room_songtitle,
        selected_hover_background = MusicRoomImages.music_room_songtitle,
        pos = (20, 0),
        top_padding = 9,
        left_padding = 18,
        bottom_padding = 5,
    ))
    style.song_list_button_text = Style(style.default, properties=dict(
        size = 22,
        idle_color = "#9CB2B0",
        hover_color = "#ffffff",
        
    ))

    style.music_room_scrollbar_vscrollbar = ScrollbarStyle(width=21, height=335)
    style.music_room_scrollbar_arrow_up = ScrollbarArrowStyle(direction='up', width=21)
    style.music_room_scrollbar_arrow_down = ScrollbarArrowStyle(direction='down', width=21)